function biasest()
%Bootstrap routine for bias estimation

%Get handles
stath=findobj(gcbf,'Tag','PopupMenu1');
disth=findobj(gcbf,'Tag','PopupMenu3');
nh=findobj(gcbf,'Tag','EditText_N');
if ~chkno(nh,'The number of samples')
  return;
end
Bh=findobj(gcbf,'Tag','EditText_B1');
if ~chkno(Bh,'B1')
  return;
end
MCh=findobj(gcbf,'Tag','EditText_MC');
if ~chkno(MCh,'The number of Monte Carlo runs')
  return;
end
outputh=findobj(gcbf,'Tag','Axes1');
waitbarh=findobj(gcbf,'Tag','Waitbar');
applybtnh=findobj(gcbf,'Tag','Pushbutton1');

%Get Matlab expressions from popup menus
matlabexp=get(stath,'UserData');
statfun=matlabexp{get(stath,'Value')};
matlabexp=get(disth,'UserData');
distfun=matlabexp{get(disth,'Value')};

%Get settings from edit boxes
B=str2num(get(Bh,'String'));
n=str2num(get(nh,'String'));
MC=str2num(get(MCh,'String'));

%Set bootstrap parameter function
stat=get(stath,'String');
if strcmp(stat{get(stath,'Value')},'variance (unbiased)')
  prmfun='var(X,1)';
else
  prmfun=statfun;
end

%Change Apply button to Cancel button
applycb=get(applybtnh,'Callback');
set(applybtnh, 'String','Cancel', ...
  'Callback',['set(findobj(gcbf,''Tag'',''Pushbutton1''),''String'',''Apply'',''Callback'',''' applycb ''');']);

%Run bootstrap procedure
x=eval(distfun); %e.g. distfun='randn(n,MC)'
bias=zeros(1,MC);
if strcmp(statfun,'corr(X)') %kludge for bivariate data
  p=2;
else
  p=1;
end
Xstar=zeros(n,B,p);
i=1;
while i<=MC & strcmp(get(applybtnh,'String'),'Cancel')
  X=x(:,i,:);
  bootprm=eval(prmfun); %e.g. prmfun='mean(X)'
  Xstar(:)=x(ceil(n*rand(n,B)),i,:);
  X=Xstar;
  bootstat=eval(statfun); %e.g. statfun='mean(X)'
  bias(i)=mean(bootstat)-bootprm;
  set(waitbarh,'XData',[0 i/MC i/MC 0]);drawnow;
  i=i+1;
end
set(waitbarh,'XData',[0 0 0 0]);

if i<=MC %Cancel button was pressed
  return; %do not output results
end

%Output results
fontsize=get(outputh,'FontSize');
fontweight=get(outputh,'FontWeight');
axes(outputh); cla; grid off;
if MC==1
  hist(bootstat,20);
  title(sprintf('Bootstrap Histogram (Bias= %.2f)',bias));
  xlabel('Bootstrap statistics');
  ylabel('Frequency of occurrence');
else
  hist(bias,20);
  title(sprintf('Monte Carlo Histogram (Average bias=%.2f)',mean(bias)));
  xlabel('Bootstrap bias estimates');
  ylabel('Frequency of occurrence');
end
%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);

%Restore Apply button
set(applybtnh,'String','Apply','Callback',applycb);
