function biastut()
%Bootstrap tutorial routine for bias estimation

%Get handles
stath=findobj(gcbf,'Tag','PopupMenu1');
disth=findobj(gcbf,'Tag','PopupMenu3');
nh=findobj(gcbf,'Tag','EditText_N');
if ~chkno(nh,'The number of samples')
  return;
end
Bh=findobj(gcbf,'Tag','EditText_B1');
if ~chkno(Bh,'B1')
  return;
end
MCh=findobj(gcbf,'Tag','EditText_MC');
if ~chkno(MCh,'The number of Monte Carlo runs')
  return;
end
outputh=findobj(gcbf,'Tag','Axes1');
waitbarh=findobj(gcbf,'Tag','Waitbar');
startbtnh=findobj(gcbf,'Tag','Pushbutton1');

%Get Matlab expressions from popup menus
matlabexp=get(stath,'UserData');
statfun=matlabexp{get(stath,'Value')};
matlabexp=get(disth,'UserData');
distfun=matlabexp{get(disth,'Value')};

%Get settings from edit boxes
B=str2num(get(Bh,'String'));
n=str2num(get(nh,'String'));
MC=str2num(get(MCh,'String'));

%Set bootstrap parameter function
stat=get(stath,'String');
if strcmp(stat{get(stath,'Value')},'variance (unbiased)')
  prmfun='var(X,1)';
else
  prmfun=statfun;
end

%Change Start button to Continue button
startcb=get(startbtnh,'Callback');
set(startbtnh, 'String','Continue', 'Callback','uiresume(gcbf)');

%Run bootstrap procedure

%Output results as we go
fontsize=get(outputh,'FontSize');
fontweight=get(outputh,'FontWeight');
axes(outputh); cla; title(''); axis off;

axes(outputh);
text(0.05,0.95,'Step 0. Experiment: Collect data and estimate parameter.','Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
X=eval(distfun); %e.g. distfun='randn(n,MC)'
if n>10
  s=['X = (' sprintf('%.2f,',X(1:9)) '...)'];
else
  s=['X = (' sprintf('%.2f,',X(1:end-1)) sprintf('%.2f',X(end)) ')'];
end
text(0.1,0.9,s,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
bootprm=eval(prmfun);
text(0.1,0.85,sprintf('{\\it P} = %g',bootprm),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.2 0.2 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.75,'Step 1. Resampling: Draw resample with replacement from data.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
X=X(ceil(n*rand(n,B)));
if n>10
  s=['X* = (' sprintf('%.2f,',X(1:9)) '...)'];
else
  s=['X* = (' sprintf('%.2f,',X(1:n-1)) sprintf('%.2f',X(n)) ')'];
end
text(0.1,0.7,s,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.4 0.4 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.6,'Step 2. Calculation: Calculate bootstrap estimate from resample.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
bootstat=eval(statfun); %e.g. statfun='mean(X)'
text(0.1,0.55,sprintf('{\\it P}* = %g',bootstat(1)),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.6 0.6 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.45,'Step 3. Repetition: Repeat Steps 1 and 2 for B1 bootstrap estimates.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
if B>10
  s=['P* = (' sprintf('%.2f,',bootstat(1:9)) '...)'];
else
  s=['P* = (' sprintf('%.2f,',bootstat(1:end-1)) sprintf('%.2f',bootstat(end)) ')'];
end
text(0.1,0.4,s,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.8 0.8 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.3,'Step 4. Bias Estimation: Estimate bias by (1/B1) \Sigma {\it P}*(i) - {\it P}.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
bias=mean(bootstat)-bootprm;
text(0.1,0.25,sprintf('bias = %.2f',bias),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 1 1 0]);
set(startbtnh, 'String','End');
uiwait(gcbf);

set(waitbarh,'XData',[0 0 0 0]);

%Output results
axes(outputh);
hist(bootstat,20);
title(sprintf('Bootstrap Histogram (Bias= %.2f)',bias));
xlabel('Bootstrap statistics');
ylabel('Frequency of occurrence');

%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);

%Restore Start button
set(startbtnh, 'String','Start', 'Callback',startcb);
