function ciestvs()
%Bootstrap routine for confidence interval estimation (Variance Stabilisation)

%Get handles
stath=findobj(gcbf,'Tag','PopupMenu1');
disth=findobj(gcbf,'Tag','PopupMenu3');
nh=findobj(gcbf,'Tag','EditText_N');
if ~chkno(nh,'The number of samples')
  return;
end
B1h=findobj(gcbf,'Tag','EditText_B1');
if ~chkno(B1h,'B1')
  return;
end
B2h=findobj(gcbf,'Tag','EditText_B2');
if ~chkno(B2h,'B2')
  return;
end
B3h=findobj(gcbf,'Tag','EditText_B3');
if ~chkno(B3h,'B3')
  return;
end
MCh=findobj(gcbf,'Tag','EditText_MC');
if ~chkno(MCh,'The number of Monte Carlo runs')
  return;
end
alphah=findobj(gcbf,'Tag','EditText_alpha');
if ~chkalpha(alphah)
  return;
end
outputh=findobj(gcbf,'Tag','Axes1');
waitbarh=findobj(gcbf,'Tag','Waitbar');
applybtnh=findobj(gcbf,'Tag','Pushbutton1');

%Get Matlab expressions from popup menus
matlabexp=get(stath,'UserData');
statfun=matlabexp{get(stath,'Value')};
matlabexp=get(disth,'UserData');
distfun=matlabexp{get(disth,'Value')};

%Get settings from edit boxes
B1=str2num(get(B1h,'String'));
B2=str2num(get(B2h,'String'));
B3=str2num(get(B3h,'String'));
n=str2num(get(nh,'String'));
MC=str2num(get(MCh,'String'));
alpha=str2num(get(alphah,'String'));

%Change Apply button to Cancel button
applycb=get(applybtnh,'Callback');
set(applybtnh, 'String','Cancel', ...
  'Callback',['set(findobj(gcbf,''Tag'',''Pushbutton1''),''String'',''Apply'',''Callback'',''' applycb ''');']);

%Run bootstrap procedure
q1=floor(B3*alpha/2);
if q1<1
  q1=1;
  B3=ceil(2/alpha);
  h=warndlg(sprintf('Minimum number of bootstrap replications required for the significance level chosen is %d.',B3));
  set(h,'WindowStyle','modal');
  uiwait(h);
  set(B3h,'String',num2str(B3));
end
q2=B3-q1+1;
x=eval(distfun); %e.g. distfun='randn(n,MC)'
lower=zeros(1,MC);
upper=zeros(1,MC);
vstar=zeros(1,B1);
if strcmp(statfun,'corr(X)') %kludge for bivariate data
  p=2;
else
  p=1;
end
Xstar=zeros(n,B1,p);
Xstar2=zeros(n,B2,p);
Xstar3=zeros(n,B3,p);
i=1;
while i<=MC & strcmp(get(applybtnh,'String'),'Cancel')

  %Algorithm from Tibshirani (1988)

  X=x(:,i,:);
  S=eval(statfun);

  %Step 1. Compute the variance stabilizing transformation as follows.

  % (a) Generate B1 bootstrap samples Xstar(1:B1) and
  %     for each calculate the value of the statistic Sstar(1:B1).
  Xstar(:)=x(ceil(n*rand(n,B1)),i,:);
  X=Xstar;
  Sstar=eval(statfun);

  % (b) For b=1,...,B1, generate B2 bootstrap samples from the estimate
  %     of the population based on Xstar(b) and thus compute vstar(b)
  %     the bootstrap estimate of variance of Sstar(b).
  b=1;
  while b<=B1 & strcmp(get(applybtnh,'String'),'Cancel')
    Xstar2(:)=Xstar(ceil(n*rand(n,B2)),b,:);
    X=Xstar2;
    vstar(b)=var(eval(statfun));
    if mod(b,100)==0
      j=((i-1)*B1+b)/(MC*B1);set(waitbarh,'XData',[0 j j 0]);drawnow;
    end
    b=b+1;
  end
  if b<=B1 %Cancel button was pressed
    break;
  end

  % (c) Estimate the variance function v(s) by smoothing the values of vstar
  %     against Sstar.
  [s,vstar,v]=smooth(Sstar,log(vstar),0.5);
  vstar=exp(vstar);
  v=exp(v);

  % (d) Compute the variance stabilizing transformation h(theta).
  f=v.^(-0.5);
  h=cumsum([0 0.5*(f(1:end-1)+f(2:end)).*diff(s)]);

  %Step 2. Compute a bootstrap t interval for h(theta): generate B3 bootstrap
  %samples and compute Sstar(1:B3) and thus h(Sstar(b)) for each.  The
  %alpha-th critical point for h(theta) is given by h(S)-y(1-alpha), where
  %y(alpha) is the alpha-th percentile of the bootstrap distribution of
  %h(Sstar)-h(S).
  Xstar3(:)=x(ceil(n*rand(n,B3)),i,:);
  X=Xstar3;
  Sstar=eval(statfun);
  hS=linterp(s,h,S);
  y=sort(linterp(s,h,Sstar)-hS);
  
  %Step 3. Compute the interval for theta: back-transform the interval for
  %h(theta) to give the alpha point h^(-1){h(S)-y(1-alpha)} for theta.
  lower(i)=linterp(h,s,hS-y(q2));
  upper(i)=linterp(h,s,hS-y(q1));

  set(waitbarh,'XData',[0 i/MC i/MC 0]);drawnow;
  i=i+1;
end
set(waitbarh,'XData',[0 0 0 0]);

if i<=MC %Cancel button was pressed
  return; %do not output results
end

%Output results
fontsize=get(outputh,'FontSize');
fontweight=get(outputh,'FontWeight');
axes(outputh); cla; grid off;
if MC==1
  hist(y,20);
  title(sprintf('Bootstrap Histogram (C.I.= [%.2f,%.2f])',lower,upper));
  xlabel('Bootstrap statistics');
  ylabel('Frequency of occurrence');
%  subplot(3,1,2);
%  plot(s,sqrt(vstar),'*');
%  xlabel('Bootstrap statistics');
%  ylabel('Std dev.');
%  subplot(3,1,3);
%  plot(s,h,'*');
%  xlabel('Parameter');
%  ylabel('Transformation');
else
  plot(1:MC,[lower;upper]);
  grid on;
  title(sprintf('Bootstrap Confidence Intervals (Average C.I.= [%.2f,%.2f])',mean(lower),mean(upper)));
  xlabel('Monte Carlo run');
  ylabel('Parameter value');
end
%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);

%Restore Apply button
set(applybtnh,'String','Apply','Callback',applycb);


function [x_sort, y_sort, y_sm] = smooth(x, y, w)
%          [x_sort, y_sort, y_sm] = smooth(x, y, w);
%          A running line smoother that fits the data by linear 
%          least squares. Used to compute the variance stabilising 
%          transformation.
%
%    Inputs:  
%          x - one or more columns of covariates
%          y - one column of response for each column of covariate
%          w - span, proportion of data in symmetric centred window
%              
%   Outputs: 
%     x_sort - sorted columns of x
%     y_sort - values of y associated with x_sort
%       y_sm - smoothed version of y
%
%  Note: If inputs are row vectors, operation is carried out row-wise.

%  Created by  Hwa-Tung Ong, 1996
%
% References
%
% Hastie, T.J. and Tibshirani, R.J. Generalised additive models.
%                Chapman and Hall, 1990.  
%
% Zoubir, A.M.   Bootstrap: Theory and Applications. Proceedings 
%                of the SPIE 1993 Conference on Advanced  Signal 
%                Processing Algorithms, Architectures and Imple-
%                mentations. pp. 216-235, San Diego, July  1993.
%
%  Zoubir, A.M. and Boashash, B. The Bootstrap: Signal Processing 
%               Applications. IEEE Transactions  on  Signal  Pro-
%               cessing, Vol. 15, No. 1, pp. 55-76, 1998.

if any(size(x) ~= size(y))
    error('Input matrices must be the same length.'),
end

[nr,nc] = size(x);
n=nr;
if (nr==1) x=x';y=y';n=nc;nc=1; end
y_sm = zeros(n,nc);
[x_sort,order] = sort(x);
for i = 1:nc y_sort(:,i) = y(order(:,i),i); end
k = fix(w*n/2);

for i = 1:n
    window = max(i-k,1):min(i+k,n);
    xwin = x_sort(window,:);
    ywin = y_sort(window,:);
    xbar = mean(xwin);
    ybar = mean(ywin);
    copy = ones(length(window),1);
    x_mc = xwin - copy*xbar;    % mc = mean-corrected
    y_mc = ywin - copy*ybar;
    y_sm(i,:) = sum(x_mc.*y_mc)./sum(x_mc.*x_mc) .* (x_sort(i,:)-xbar) + ybar;
end

if (nr==1) x_sort=x_sort';y_sort=y_sort';y_sm=y_sm'; end


function y0 = linterp(x, y, x0)
%linterp   Linear interpolation
%
%Description:
%          A Matlab function to map given abscissa values
%          to ordinates, performing linear interpolation
%          and extrapolation when the input table of data
%          does not provide the right coordinates.
%
%Usage:    y0 = linterp(x, y, x0);
%
%Inputs:   x       one or more sorted columns of abscissa values
%          y       corresponding columns of ordinates
%          x0      abscissa values to map in corresponding columns
%
%Outputs:  y0      interpolated values corresponding to x0
%
%Note:     If inputs are row vectors, operation is carried out row-wise.
%
%Author:   Hwa-Tung Ong, 1996

% check inputs
if any(size(x) ~= size(y))
    error('Input matrices X and Y must be the same size.')
end
[nr,nc] = size(x);
n=nr;
if (nr==1) x=x';y=y';x0=x0';n=nc;nc=1; end
np = size(x0,1);
if nc ~= size(x0,2)
    error('Input matrices must have the same number of columns.')
end

% map x0 to y0
y0 = zeros(np,nc);
for c = 1:nc

    % find index of closest x on the left side of x0
    k = sum(x(:,c)*ones(1,np) < ones(n,1)*x0(:,c)');

    % adjust index for x0 outside x range
    i = find(k == 0);
    k(i) = ones(size(i));
    i = find(k == n);
    k(i) = ones(size(i))*(n-1);

    % linear interpolate or extrapolate to get y0
    y0(:,c) = (y(k+1,c)-y(k,c))./(x(k+1,c)-x(k,c)) .* (x0(:,c)-x(k,c)) + y(k,c);
end

if (nr==1) y0=y0'; end
