function cinpbtut()
%Bootstrap tutorial routine for confidence interval estimation (non-parametric bootstrap)

%Get handles
stath=findobj(gcbf,'Tag','PopupMenu1');
disth=findobj(gcbf,'Tag','PopupMenu3');
nh=findobj(gcbf,'Tag','EditText_N');
if ~chkno(nh,'The number of samples')
  return;
end
Bh=findobj(gcbf,'Tag','EditText_B1');
if ~chkno(Bh,'B1')
  return;
end
MCh=findobj(gcbf,'Tag','EditText_MC');
if ~chkno(MCh,'The number of Monte Carlo runs')
  return;
end
alphah=findobj(gcbf,'Tag','EditText_alpha');
if ~chkalpha(alphah)
  return;
end
outputh=findobj(gcbf,'Tag','Axes1');
waitbarh=findobj(gcbf,'Tag','Waitbar');
startbtnh=findobj(gcbf,'Tag','Pushbutton1');

%Get Matlab expressions from popup menus
matlabexp=get(stath,'UserData');
statfun=matlabexp{get(stath,'Value')};
matlabexp=get(disth,'UserData');
distfun=matlabexp{get(disth,'Value')};

%Get settings from edit boxes
B=str2num(get(Bh,'String'));
n=str2num(get(nh,'String'));
MC=str2num(get(MCh,'String'));
alpha=str2num(get(alphah,'String'));

%Change Start button to Continue button
startcb=get(startbtnh,'Callback');
set(startbtnh, 'String','Continue', 'Callback','uiresume(gcbf)');

%Run bootstrap procedure

%Output results as we go
fontsize=get(outputh,'FontSize');
fontweight=get(outputh,'FontWeight');
axes(outputh); cla; title(''); axis off;

q1=floor(B*alpha/2);
if q1<1
  q1=1;
  B=ceil(2/alpha);
  h=warndlg(sprintf('Minimum number of bootstrap replications required for the significance level chosen is %d.',B));
  set(h,'WindowStyle','modal');
  uiwait(h);
  set(Bh,'String',num2str(B));
end
q2=B-q1+1;

axes(outputh);
text(0.05,0.95,'Step 0. Experiment: Collect data.','Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
X=eval(distfun); %e.g. distfun='randn(n,MC)'
if n>10
  s=['X = (' sprintf('%.2f,',X(1:9)) '...)'];
else
  s=['X = (' sprintf('%.2f,',X(1:end-1)) sprintf('%.2f',X(end)) ')'];
end
text(0.1,0.9,s,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.15 0.15 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.8,'Step 1. Resampling: Draw resample with replacement from data.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
X=X(ceil(n*rand(n,B)));
if n>10
  s=['X* = (' sprintf('%.2f,',X(1:9)) '...)'];
else
  s=['X* = (' sprintf('%.2f,',X(1:n-1)) sprintf('%.2f',X(n)) ')'];
end
text(0.1,0.75,s,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.3 0.3 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.65,'Step 2. Calculation: Calculate bootstrap estimate from resample.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
bootstat=eval(statfun); %e.g. statfun='mean(X)'
text(0.1,0.6,sprintf('{\\it P}* = %g',bootstat(1)),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.45 0.45 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.5,'Step 3. Repetition: Repeat Steps 1 and 2 for B1 bootstrap estimates.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
if B>10
  s=['P* = (' sprintf('%.2f,',bootstat(1:9)) '...)'];
else
  s=['P* = (' sprintf('%.2f,',bootstat(1:end-1)) sprintf('%.2f',bootstat(end)) ')'];
end
text(0.1,0.45,s,'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.65 0.65 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.35,'Step 4. Approximation of Distribution: Sort bootstrap estimates.','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
bootstat=sort(bootstat);
if B>10
  s=['P* = (' sprintf('%.2f,',bootstat(1:8)) sprintf('...,%.2f)',bootstat(end))];
else
  s=['P* = (' sprintf('%.2f,',bootstat(1:end-1)) sprintf('%.2f',bootstat(end)) ')'];
end
text(0.1,0.3,s,'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 0.85 0.85 0]);
uiwait(gcbf);

axes(outputh);
text(0.05,0.2,'Step 5. Confidence Interval: The 100(1-\alpha)% c. i. is [{\it P}*(q_1),{\it P}*(q_2)].','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
text(0.1,0.15,sprintf('q_1 = [B1\\alpha/2] = %d',q1),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
text(0.1,0.1,sprintf('q_2 = B1-q_1+1 = %d',q2),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
lower=bootstat(q1);
upper=bootstat(q2);
text(0.1,0.05,sprintf('C.I. = [%.2f,%.2f]',lower,upper),'Units','normalized','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(waitbarh,'XData',[0 1 1 0]);
set(startbtnh, 'String','End');
uiwait(gcbf);

set(waitbarh,'XData',[0 0 0 0]);

%Output results
axes(outputh);
hist(bootstat,20);
title(sprintf('Bootstrap Histogram (C.I.= [%.2f,%.2f])',lower,upper));
xlabel('Bootstrap statistics');
ylabel('Frequency of occurrence');

%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);

%Restore Start button
set(startbtnh,'String','Start','Callback',startcb);
