function d2scb()
%Bootstrap routine for two-sensor signal detection

%Get handles
signalh=findobj(gcbf,'Tag','PopupMenu1');
bkgndh=findobj(gcbf,'Tag','PopupMenu2');
sensorh=findobj(gcbf,'Tag','PopupMenu3');
nh=findobj(gcbf,'Tag','EditText_N');
if ~chkno(nh,'The number of samples')
  return;
end
Bh=findobj(gcbf,'Tag','EditText_B1');
if ~chkno(Bh,'B1')
  return;
end
MCh=findobj(gcbf,'Tag','EditText_MC');
if ~chkno(MCh,'The number of Monte Carlo runs')
  return;
end
alphah=findobj(gcbf,'Tag','EditText_alpha');
if ~chkalpha(alphah)
  return;
end
outputh=findobj(gcbf,'Tag','Axes1');
waitbarh=findobj(gcbf,'Tag','Waitbar');
applybtnh=findobj(gcbf,'Tag','Pushbutton1');

%Get Matlab expressions from popup menus
matlabexp=get(signalh,'UserData');
choice=get(signalh,'Value');
signalfun=matlabexp{choice};
if choice==length(get(signalh,'String')) %use the user-provided delay
  signald=matlabexp{end}; %signal time delay
else
  signald=10;
end
matlabexp=get(bkgndh,'UserData');
choice=get(bkgndh,'Value');
bkgndfun=matlabexp{choice};
if choice==length(get(bkgndh,'String')) %use the user-provided delay
  bkgndd=matlabexp{end}; %background noise time delay
else
  bkgndd=5;
end
matlabexp=get(sensorh,'UserData');
sensorfun=matlabexp{get(sensorh,'Value')};

%Get settings from edit boxes
B=str2num(get(Bh,'String'));
n2=str2num(get(nh,'String'));
MC=str2num(get(MCh,'String'));
alpha=str2num(get(alphah,'String'));

%Change Apply button to Cancel button
applycb=get(applybtnh,'Callback');
set(applybtnh, 'String','Cancel', ...
  'Callback',['set(findobj(gcbf,''Tag'',''Pushbutton1''),''String'',''Apply'',''Callback'',''' applycb ''');']);

%Run bootstrap procedure
n=n2+signald;
s=eval(signalfun);
n=n2+bkgndd;
w=eval(bkgndfun);
n=n2;
w1=eval(sensorfun);
w2=eval(sensorfun);
x=s(1+signald:end,:)+w(1+bkgndd:end,:)+w1;
y=s(1:end-signald,:)+w(1:end-bkgndd,:)+w2;
[T,lambda,T_ast]=bootteststat(x,y,floor(n2^(4/9)),B,alpha,waitbarh,applybtnh);
if isempty(T) %Cancel button was pressed
  set(waitbarh,'XData',[0 0 0 0]);
  return; %do not output results
end
Pd=mean(T>lambda);
if get(signalh,'Value')==1 %no signal
  SNR='-\infty';
elseif get(bkgndh,'Value')==1 & get(sensorh,'Value')==1 %no noise
  SNR='\infty';
else
  SNR=num2str(round(10*log10(var(s(:))/(var(w(:))+var([w1(:);w2(:)])))));
end
set(waitbarh,'XData',[0 0 0 0]);

%Output results
fontsize=get(outputh,'FontSize');
fontweight=get(outputh,'FontWeight');
axes(outputh); cla; grid off;
if MC==1
  hist(T_ast,20);
  hold on;
  a=axis;
  plot([lambda lambda],a(3:4),'r');
  plot([T T],a(3:4),'g');
  hold off;
  title(['Bootstrap Histogram (red: threshold, green: test statistic, SNR~ ' SNR ' dB)']);
  xlabel('Bootstrap statistics');
  ylabel('Frequency of occurrence');
else
  plot(1:MC,lambda,'r',1:MC,T,'g');
  grid on;
  title(sprintf('red: threshold, green: test statistic (%%alarms= %d%%, SNR~ %s dB)',round(100*Pd),SNR));
  xlabel('Monte Carlo run');
  ylabel('Bootstrap statistic value');
end
%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);

%Restore Apply button
set(applybtnh,'String','Apply','Callback',applycb);


function [T,lambda,T_b] = bootteststat(X,Y,N,B,alpha,waitbarh,applybtnh)
%[T,lambda] = bootteststat(x,y,N,B,alpha,waitbarh);
%Test statistic for bootstrap detection of
%common non-Gaussian signals at two sensors
%
% x,y  - time series in columns
%  N   - number of samples in a segment
%  B   - number of bootstrap samples
%alpha - nominal level of the test
%waitbarh - graphics handle for waitbar
%  T   - test statistic
%lambda- bootstrap threshold
%  T_b - last set of bootstrap statistics

% [18 Nov 97] Based on conGtest3.
% [22 Jan 99] Adapted for bootstrap tutorial.

dim = size(X);

% Segment data
K = fix(dim(1)/N); %number of segments of size N
if dim(1)~=K*N,
  X = X(1:K*N,:);
  Y = Y(1:K*N,:);
end
R = prod(dim(2:end));
X = fft(reshape(X,[N,K,R]));
Y = fft(reshape(Y,[N,K,R]));
T = zeros([1 dim(2:end)]);
lambda = T;
T_b = zeros(1,B);
q = floor((1-alpha)*(B+1));
[m,n,L] = pd3(N); %originally pd3(N,'cross noreal')
k = repmat(0:K-1,L,1);
mplusn = mod(m+n,N)+1;
m = m+1; n = n+1;
k1 = fix(sqrt(N)/2);
mplusn2 = mplusn+k1;
m2 = m+k1; n2 = n+k1;
win = bpwindow(2*k1+1,1);
N_etc = repmat(N*(1+(m==n)),1,K);

i=1;
while i<=R & strcmp(get(applybtnh,'String'),'Cancel')

  % Find cross biperiodogram and estimate variance
  I = X(m,:,i).*X(n,:,i).*conj(Y(mplusn,:,i))/N;
  B_hat = mean(I,2);
  Sx = filter(win,1,[abs(X(:,:,i)).^2/N; zeros(k1,K)]);
  Sy = filter(win,1,[abs(Y(:,:,i)).^2/N; zeros(k1,K)]);
  V = N_etc.*Sx(m2,:).*Sx(n2,:).*Sy(mplusn2,:);

  % Find residuals
  epsilon_hat = (I - B_hat(:,ones(1,K)))./sqrt(V);
  epsilon_hat = epsilon_hat - repmat(mean(epsilon_hat),L,1);

  % Find test statistic
  T(i) = sum(abs(B_hat)./sqrt(V*mean(epsilon_hat.*conj(epsilon_hat))'));

  % Work out bootstrap threshold
  sqrtV = sqrt(V);
  b=1;
  while b<=B & strcmp(get(applybtnh,'String'),'Cancel')
    I_b = epsilon_hat(ceil(rand(L,K)*L)+k);
    T_b(b) = sum(abs(mean(sqrtV.*I_b,2))./sqrt(V*real(var(I_b,1))'));
    if mod(b,10)==0
      j=((i-1)*B+b)/(B*R);set(waitbarh,'XData',[0 j j 0]);drawnow;
    end
    b=b+1;
  end
  if b<=B %Cancel button was pressed
    break;
  end
  T_b = sort(T_b);
  lambda(i) = T_b(q);

  set(waitbarh,'XData',[0 i/R i/R 0]);drawnow;
  i=i+1;
end
if b<=B | i<=R %Cancel button was pressed
  T=[];
end


function [m,n,L] = pd3(N)
%[m,n,L] = pd3(N);
%Returns the discrete frequency pairs (m,n) in the principal domain of
%the third-order spectrum for data of length N.  The number of pairs
%are returned in L.
%  'cross noreal'	- for cross-bispectrum complex elements only
%The principal domain of the cross-bispectrum is composed of
%   {(m,n)| 0 <= n <= m < N/2} and {(m,n)| N-m <= n < N, m < N/2},
%two triangles with apexes at (0,0), (N/2,0) and (N/2,N/2), and
%(N/2,N/2), (0,N) and (N/2,N).

N = N(1);		% N must be scalar
if N<1			% catch degenerate cases
  m=[]; n=[]; L=0;
  return;
end
N2 = 0.5*N;		% calculate constant N/2 once only
N3 = N/3;		% calculate constant N/3 once only
IN2I = fix(N2);		% calculate constant fix(N/2) once only
skip1 = 0;
skip2 = 0;
% Work out first and last m for n=0,...,N-1
  % catch special cases
  if N<3 m=[];n=[];L=0;return; end

  mstart = [1 1:ceil(N2-1) IN2I+1:-1:2];
  mstop = IN2I;
  skip1 = ~rem(N,2);	% need to skip real point (N/2,0)

% L is the number of elements in the principal domain
L = N*(mstop+1)-sum(mstart)-skip1;

% Form discrete frequency pairs (m,n) going along m for each n
m = zeros(L,1);
n = zeros(L,1);
if skip1	% then do the first iteration separately to exclude (N/2,0)
  ind = 1:N2-1;
  m(ind) = ind;
  startp = N2;
else
  startp = 1;
end
for k = skip1:N-1
  endp = startp+(mstop-mstart(k+1));
  ind = startp:endp;
  m(ind) = mstart(k+1):mstop;
  n(ind) = n(ind)+k+skip2;
  startp = endp+1;
end

function m = mod(x,y)
m = x - floor(x./y).*y;

function w = bpwindow(N,M)
%w = bpwindow(N,M);
%N is the number of data points for the Barlett-Priestley spectral window
%M is the truncation point
win = 1-[0:2*M/N:1].^2;
w = [win(end:-1:2) win];
w = w/sum(w);
