function modelsel()
%Bootstrap routine for confidence interval estimation (Percentile-t)

%Get handles
outputh=findobj(gcbf,'Tag','Axes1');
modelh=findobj(gcbf,'Tag','PopupMenu1');
disth=findobj(gcbf,'Tag','PopupMenu3');
nh=findobj(gcbf,'Tag','EditText_N');
if ~chkno(nh,'The number of samples')
  return;
end
Bh=findobj(gcbf,'Tag','EditText_B1');
if ~chkno(Bh,'B')
  return;
end
SNRh=findobj(gcbf,'Tag','EditText_B2');

Qh=findobj(gcbf,'Tag','EditText_B3');
if ~chkno(Bh,'Q')
  return;
end
MCh=findobj(gcbf,'Tag','EditText_MC');
if ~chkno(MCh,'The number of Monte Carlo runs')
  return;
end

outputh=findobj(gcbf,'Tag','Axes1');
waitbarh=findobj(gcbf,'Tag','Waitbar');
applybtnh=findobj(gcbf,'Tag','Pushbutton1');

%Get Matlab expressions from popup menus
matlabexp=get(modelh,'UserData');
model=matlabexp{get(modelh,'Value')};
matlabexp=get(disth,'UserData');
distfun=matlabexp{get(disth,'Value')};
%order=str2num(get(orderh,'String'));
%Get settings from edit boxes
B=str2num(get(Bh,'String'));
n=str2num(get(nh,'String'));
MC=str2num(get(MCh,'String'));
SNR=str2num(get(SNRh,'String'));
Q=str2num(get(Qh,'String'));

%Change Apply button to Cancel button
applycb=get(applybtnh,'Callback');
set(applybtnh, 'String','Cancel', ...
  'Callback',['set(findobj(gcbf,''Tag'',''Pushbutton1''),''String'',''Apply'',''Callback'',''' applycb ''');']);

%Run bootstrap procedure

fontsize=get(outputh,'FontSize');
axes(outputh); cla; title(''); axis off;

t=(0:n-1)'; m=48;
signalphase=eval(model);
x=eval(distfun); %e.g. distfun='randn(n,MC)'
x=(10^(-SNR/20))*x;
IND=zeros(ceil((2^(Q+1)-1)/2),5);
i=1;
while i<=MC & strcmp(get(applybtnh,'String'),'Cancel')
   X = exp(j*signalphase)+x(:,i);
   [G,aic,mdl,hq,aicc,models] = polymodel(X,B,Q,m);                   
   [Min,ind]=min(G);
   IND(ind,1)=IND(ind,1)+1;
   [Min,ind]=min(aic);
   IND(ind,2)=IND(ind,2)+1;
   [Min,ind]=min(mdl);
   IND(ind,3)=IND(ind,3)+1;
   [Min,ind]=min(hq);
   IND(ind,4)=IND(ind,4)+1;
   [Min,ind]=min(aicc);
   IND(ind,5)=IND(ind,5)+1;
   set(waitbarh,'XData',[0 i/MC i/MC 0]);drawnow;
   i=i+1;
end
set(waitbarh,'XData',[0 0 0 0]);

if i<=MC %Cancel button was pressed
  return; %do not output results
end

%Output results

fontsize=get(outputh,'FontSize');
fontweight=get(outputh,'FontWeight');
axes(outputh);
text(0.15,0.5,'Model Selection Results Displayed in a Separate Figure','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
figure
subplot(211);pltmat_color(models','Models','k','k',fontsize,fontweight)
if Q==3,
   ylabel('a_3     a_2     a_1     a_0','FontWeight',fontweight)
elseif Q==4,
   ylabel('a_4   a_3   a_2   a_1   a_0','FontWeight',fontweight)
end
subplot(212);pltmat_color(IND','Model Selection Results','k','k',fontsize,fontweight)
ylabel('AICC   HQ   MDL   AIC   Boot','FontWeight',fontweight)
%Ensure crucial properties are not changed after plotting
set(outputh,'Tag','Axes1','FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'title'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'xlabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);
set(get(outputh,'ylabel'),'FontUnits','normalized','FontSize',fontsize,'FontWeight',fontweight);

%Restore Apply button
set(applybtnh,'String','Apply','Callback',applycb);

%%%%%%%%%%%%%%%% Subfunctions %%%%%%%%%%%%%%%%%%%%
function pltmat_color(a,matrixName,gridColor,fontColor,fontSize,fontWeight);
%PLTMAT Display a matrix in a figure window.
%   PLTMAT(X,matrixName,gridColor,fontColor,fontSize,fontWeight)
%   takes the matrix X and displays it using gridColor for the
%   XColor and YColor, fontSize for the FontSize of each
%   matrix element, fontWeight for the FontWeight of each
%   matrix element and matrixName as the title for the plot. 

%   Ned Gulley, Denise Chen; June 1993.
%   Copyright (c) 1984-98 by The MathWorks, Inc.
%   $Revision: 5.5 $  $Date: 1997/11/21 23:26:47 $

% Now this function allows to change the font color as well.
% D. R. Iskander March 1999

% Clear the current axis
cla reset;
[m,n]=size(a);

% Place the text in the correct locations
% Index over number of rows
    for rowCnt=1:m,
        % Index over number of columns
        for colCnt=1:n,
            numberString=num2str(a(rowCnt,colCnt));
            text(colCnt-.5,m-rowCnt+.5,numberString, ...
                 'HorizontalAlignment','center', ...
                 'Color',fontColor, ...
                 'FontWeight',fontWeight, ...
                 'FontSize',fontSize);
        end;
    end;

set(gca,'Box','on', ...
        'Visible','on', ...
        'xLim',[0 n], ...
        'xGrid','on', ...
        'xTickLabel',[], ...
        'xTick',0:n, ...
        'yGrid','on', ...
        'yLim',[0 m], ...
        'yTickLabel',[], ...
        'yTick',0:m, ...
        'DataAspectRatio',[1, 1, 1], ... 
        'GridLineStyle',':', ...
        'LineWidth',3, ...
        'XColor',gridColor, ...
        'YColor',gridColor);

title(matrixName,'FontSize',fontSize,'FontWeight',fontWeight);

%%%%%%%%%%%%%%LOCAL FUNCTIONS%%%%%%%%%%%%%%%%%%%%%%%%

function[G,AIC,MDL,HQ,AICC,mask] = polymodel(X,B,P,m)

N=length(X);
t=(0:N-1)';
phi=unphase(X)';
H=repmat(t,1,P+1).^repmat(0:P,N,1);
b_hat=inv(H'*H)*H'*phi;
mask=fliplr(detobi(2^P:2^(P+1)-1));
msize=length(mask);
r_hat=phi-H*b_hat;                    
r_tilde=sqrt(N/m)*(r_hat-mean(r_hat))/sqrt(1-size(mask,2)/N);          
G=zeros(1,msize);  
AIC=zeros(1,msize); 
MDL=zeros(1,msize);  
HQ=zeros(1,msize); 
AICC=zeros(1,msize); 

for ii=1:msize,
  ind=find(mask(ii,:)~=0);
  Hm=H(:,ind);
  b_hat_m=inv(Hm'*Hm)*Hm'*phi;
  phi_hat=Hm*b_hat_m;
  r_hat=phi-phi_hat;
  r=r_hat-mean(r_hat);
  r_hat1=(r_hat-mean(r_hat))/sqrt(1-length(ind)/N);
  AIC(ii)=log(mean(r_hat1.^2))+2*(length(ind))/N;
  MDL(ii)=log(mean(r_hat1.^2))+log(N)*(length(ind))/N;
  HQ(ii)=log(mean(r_hat1.^2))+log(log(N))*(length(ind))/N;
  AICC(ii)=log(mean(r_hat1.^2))+2*(length(ind))/(N-length(ind)-2);
  SSE=zeros(1,B);
  for b=1:B,
    r_tilde_star=boot(N,r_tilde); 
    phi_star=phi_hat(1:m)+r_tilde_star(1:m);
    Hmstar=Hm(1:m,:);
    bstarm=inv(Hmstar'*Hmstar)*Hmstar'*phi_star;
    phistar=Hm*bstarm; 
    SSE(b)=mean((phi-phistar).^2);    
  end;
  %for b=1:B,
  %  r_tilde_star=boot(N,r_tilde); 
  %  phi_star=phi_hat+r_tilde_star;
  %  bstarm=inv(Hm'*Hm)*Hm'*phi_star;
  %  phistar=Hm*bstarm; 
  %  SSE(b)=mean((phi-phistar).^2);    
  %end;
  G(ii)=mean(SSE);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function[Y]=unphase(X)
%
%  Unwrapping the phase of the signal
%     function[Y]=unphase(X)
%
T=length(X);
Re=real(X); Im=imag(X);
g=0;Y=zeros(1,T);
ii=1;
while ii<=T,
%disp(ii)
   if Re(ii)<0,
      ren=1;rep=0;
   elseif Re(ii)>0,
      rep=1;ren=0;
   end;
   if Im(ii)<0,
      imn=1;imp=0;
   elseif Im(ii)>0,
      imp=1;imn=0;
   end;
   if Re(ii)~=0,
      Y(ii)=atan(Im(ii)/Re(ii))+ren*pi+rep*imn*2*pi;
   else
      Y(ii)=imp*pi/2+imn*pi*1.5;
   end;
   if ii>1,
      Yadj=Y(ii-1)-g*2*pi;
      if (Y(ii)<Yadj-pi/2) & (Yadj>pi/2),
         g=g+1;
      end;
      if (Yadj<pi/2) & (Y(ii)>Yadj+1.5*pi),
         g=g-1;
      end;
      Y(ii)=Y(ii)+pi*g*2;
   end;
ii=ii+1;
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function b = detobi(d, n, p)
%
%
%
d = d(:);
len_d = length(d);
if min(d) < 0
    error('Cannot convert a negative number');
end;

% assign the length
if nargin < 2;
    tmp = max(d);
    b1 = [];
    while tmp > 0
        b1 = [b1 rem(tmp, 2)];
        tmp = floor(tmp/2);
    end;
    n = length(b1);
end;
if nargin < 3
    p = 2;
end;

% initial value
b = zeros(len_d, n);

% parameter assignment
for i = 1 : len_d
    j = 1; 
    tmp = d(i);
    while (j <= n) & (tmp > 0)
        b(i, j) = rem(tmp, p);
        tmp = floor(tmp/p);
        j = j + 1;
    end;
end;

